// Copyright 2013 The Flutter Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

import 'dart:math';

import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

/// A widget showing a live camera preview.
class CameraPreview extends StatelessWidget {
  /// Creates a preview widget for the given camera controller.
   const CameraPreview(this.controller, {Key? key, this.child}) : super(key: key);

  /// The controller for the camera that the preview is shown for.
  final CameraController controller;

  /// A widget to overlay on top of the camera preview
  final Widget? child;

  @override
  Widget build(BuildContext context) {
    return controller.value.isInitialized
        ? ValueListenableBuilder<CameraValue>(
            valueListenable: controller,
            builder: (BuildContext context, Object? value, Widget? child) {
              return RotatedBox(
                quarterTurns: _getQuarterTurns(),
                child: Transform.scale(
                  scale: controller.value.aspectRatio -
                      MediaQuery.of(context).size.aspectRatio,
                  child: AspectRatio(
                    aspectRatio: _isLandscape()
                        ? controller.value.aspectRatio
                        : (1 / controller.value.aspectRatio),
                    child: Stack(
                      fit: StackFit.expand,
                      children: <Widget>[
                        controller.buildPreview(),
                        child ?? Container(),
                      ],
                    ),
                  ),
                ),
              );
            },
            child: child,
          )
        : Container();
  }

  bool _isLandscape() {
    return <DeviceOrientation>[
      DeviceOrientation.landscapeLeft,
      DeviceOrientation.landscapeRight
    ].contains(_getApplicableOrientation());
  }

  int _getQuarterTurns() {
    final Map<DeviceOrientation, int> turns = <DeviceOrientation, int>{
      DeviceOrientation.portraitUp: 0,
      DeviceOrientation.landscapeRight: 1,
      DeviceOrientation.portraitDown: 2,
      DeviceOrientation.landscapeLeft: 3,
    };
    return turns[_getApplicableOrientation()]!;
  }

  DeviceOrientation _getApplicableOrientation() {
    return controller.value.isRecordingVideo
        ? controller.value.recordingOrientation!
        : (controller.value.previewPauseOrientation ??
            controller.value.lockedCaptureOrientation ??
            controller.value.deviceOrientation);
  }

  double getCameraDeviceRotate() {
    if (_getApplicableOrientation() == DeviceOrientation.landscapeLeft) {
      return 270 * pi / 180;
    } else if (_getApplicableOrientation() ==
        DeviceOrientation.landscapeRight) {
      return 90 * pi / 180;
    } else if (_getApplicableOrientation() == DeviceOrientation.portraitDown) {
      return 180 * pi / 180;
    } else {
      return 0 * pi / 180;
    }
  }
}
